/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import "../common"
import "../common/ClockUtils.js" as Utils

// Component which draws the analog alarm face.
AnalogFaceBase {
    id: alarmRoot

    property int hours: 0
    property int minutes: 0
    property alias showSetupMessage: setupMessage.visible;

    onMinutesChanged: labelAlarm.text = Utils.zeroleft(hours, 2) + ":" + Utils.zeroleft(minutes, 2);
    onHoursChanged: labelAlarm.text = Utils.zeroleft(hours, 2) + ":" + Utils.zeroleft(minutes, 2)

    // Label to show the current time
    Item {
        id: labelContainer;

        z: alarmRoot.z + 2;
        width: alarmRoot.innerCircle.width; height: width;
        anchors.centerIn: alarmRoot;

        Label {
            id: labelAlarm
            objectName: "labelAlarm"

            visible: !setupMessage.visible
            anchors.centerIn: parent
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
            color: Theme.palette.normal.baseText

            fontSize: "x-large"
            text: Utils.zeroleft(alarmRoot.hours, 2) + ":" + Utils.zeroleft(alarmRoot.minutes, 2)
        }

        Label {
            id: setupMessage
            objectName: "labelAlarmSetup"

            anchors.fill: parent
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
            color: Theme.palette.normal.baseText

            fontSize: "large"
            text: i18n.tr("Long press \nto setup \nan alarm")
            wrapMode: Text.WordWrap
        }
    }

    AnalogClockHand {
        id: hourHand
        objectName: "alarmHourHand"

        z: parent.z
        handHeight: units.gu(12.5); handWidth: units.gu(1);
        rotation: (alarmRoot.hours * 15) + (alarmRoot.minutes / 2);
    }

    AnalogClockHand {
        id: minuteHand
        objectName: "alarmMinuteHand"

        z: parent.z + 1
        handHeight: units.gu(14.5); handWidth: units.gu(0.5);
        rotation: alarmRoot.minutes * 6;
    }

}
