# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013 Canonical Ltd
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authored by: Renato Araujo Oliveira Filho <renato@canonical.com>

"""
Tests for the Clock App, main window.
"""

from __future__ import absolute_import

from testtools.matchers import Equals, NotEquals
from autopilot.matchers import Eventually

from ubuntu_clock_app.tests import ClockAppTestCase


class TestClock(ClockAppTestCase):
    """Test the clock page features."""

    def setUp(self):
        """This is needed to wait for the application to start.

        In the testfarm, the application may take some time to show up.

        """

        super(TestClock, self).setUp()
        self.assertThat(
            self.main_view.visible, Eventually(Equals(True)))

    def tearDown(self):
        super(TestClock, self).tearDown()

    def test_label_value(self):
        """The clock label must contain the correct time."""

        label = self.main_view.get_time_label()

        # TODO: investigate a way to get the system wide format for time
        # and compare the values
        self.assertThat(label.text, Eventually(NotEquals('')))

    def test_add_remove_world_location(self):
        """It must be possible to add and remove world locations."""

        self.assert_no_cities_selected()
        self.main_view.open_toolbar()
        self.switch_to_world_cities_page()

        city = self.grab_first_city()
        city_name = self.city_name_from(city)

        self.select_city(city)
        selected_city_name = self.city_name_from(self.selected_city())
        self.assertThat(selected_city_name, Eventually(Equals(city_name)))

        delete_button = self.enable_delete_button()
        self.pointing_device.click_object(delete_button)
        self.assert_no_cities_selected()

    def city_name_from(self, city):
        return city.select_many("Label")[0].text

    def enable_delete_button(self):
        selected_city = self.selected_city()
        x, y, w, h = selected_city.globalRect

        tx = x + (w / 8)
        ty = y + (h / 2)

        self.pointing_device.drag(tx, ty, w - w / 8, ty)
        delete_button = selected_city.select_many("QQuickItem")[2]
        self.assertThat(lambda: delete_button, Eventually(NotEquals(None)))

        return delete_button

    def switch_to_world_cities_page(self):
        add_city_button = self.main_view.get_add_city_button()
        self.pointing_device.click_object(add_city_button)
        self.assertThat(self.main_view.get_world_cities_page, Eventually(NotEquals(None)))

    def grab_first_city(self):
        self.assertThat(self.main_view.get_world_cities_list, Eventually(NotEquals(None)))
        cities_list = self.main_view.get_world_cities_list()
        self.assertThat(lambda: not cities_list.select_many("Base"), Eventually(Equals(False)))

        return cities_list.select_many("Base")[0]

    def select_city(self, city):
        self.pointing_device.click_object(city)
        self.assertThat(self.main_view.get_clock_page, Eventually(NotEquals(None)))

    def selected_city(self):
        self.assertThat(self.main_view.get_selected_cities_list, Eventually(NotEquals(None)))
        selected_cities = self.main_view.get_selected_cities_list()
        self.assertThat(lambda: not selected_cities.select_many("Standard"), Eventually(Equals(False)))

        return selected_cities.select_many("Standard")[0]

    def assert_no_cities_selected(self):
        self.main_view.drag_page_up("ClockPage", self)
        cities = self.main_view.get_selected_cities_list()
        self.assertThat(cities.count, Eventually(Equals(0)))
